"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _eventstoreBase = require("@resolve-js/eventstore-base");

var _getLog = require("./get-log");

var _resourceErrors = require("./resource-errors");

var _executeSequence = _interopRequireDefault(require("./execute-sequence"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const dropEvents = async pool => {
  const log = (0, _getLog.getLog)('dropEvents');
  const {
    eventsTableName,
    snapshotsTableName,
    subscribersTableName,
    connection,
    database,
    escapeId
  } = pool;
  log.debug(`dropping events`);
  log.verbose(`eventsTableName: ${eventsTableName}`);
  const eventsTableNameAsId = escapeId(eventsTableName);
  const freezeTableNameAsId = escapeId(`${eventsTableName}-freeze`);
  const threadsTableNameAsId = escapeId(`${eventsTableName}-threads`);
  const snapshotsTableNameAsId = escapeId(snapshotsTableName);
  const incrementalImportTableAsId = escapeId(`${eventsTableName}-incremental-import`);
  const subscribersTableNameAsId = escapeId(subscribersTableName);
  const statements = [`DROP TABLE IF EXISTS ${freezeTableNameAsId}`, `DROP TABLE ${threadsTableNameAsId}`, `DROP TABLE ${eventsTableNameAsId}`, `DROP TABLE ${snapshotsTableNameAsId}`, `DROP TABLE IF EXISTS ${incrementalImportTableAsId}`, `DROP TABLE ${subscribersTableNameAsId}`];
  const errors = await (0, _executeSequence.default)(connection, statements, log, error => {
    if ((0, _resourceErrors.isNotExistError)(error)) {
      return new _eventstoreBase.EventstoreResourceNotExistError(`mysql adapter for database "${database}" already dropped`);
    }

    return null;
  });
  log.debug(`finished dropping events`);
  return errors;
};

var _default = dropEvents;
exports.default = _default;
//# sourceMappingURL=drop-events.js.map